package tcp;

import util.Reader;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;
import java.util.HashMap;

public class TcpServer {
    private HashMap<Integer, TcpSocket> socketList;
    private ServerSocket server;
    private int port;

    private Reader reader;


    public TcpServer(int port) throws IOException {
        beforeInit();
        this.port = port;
        this.server = new ServerSocket(port);
        afterInit();
    }

    private void beforeInit() {
        this.socketList = new HashMap<>();
        this.reader = new Reader();
    }

    private void afterInit() {
        System.out.println("服务器启动完成，监听端口:" + port);
        new Thread(new Writer()).start();
        try {
            while (true) {
                Socket socket = server.accept();
                int id = getId();
                TcpSocket thread = new TcpSocket(this, id, socket);
                socketList.put(id, thread);
                new Thread(thread).start();
                System.out.println("Client access<" + id + ":" + socket.getRemoteSocketAddress() + ">");
            }
        } catch (SocketException e) {
            System.out.println("服务器关闭");
        } catch (IOException e) {
            System.out.println("无法获取输入输出流");
        }
    }

    private synchronized int getId() {
        for (int i = 1; i < 10000; i++) {
            if (!socketList.containsKey(i)) {
                return i;
            }
        }
        return -1;
    }

    public void removeClient(TcpSocket socket) {
        socketList.remove(socket.getId());
        System.out.println("与客户端<" + socket.getId() + ":" + socket.getSocket().getRemoteSocketAddress() + ">失去连接");
    }

    private class Writer implements Runnable {

        @Override
        public void run() {
            String line;
            System.out.println("start");
            while ((line = reader.readKB()) != null) {
                try {
                    int idx = line.indexOf(":");
                    if (idx == -1) {
                        System.out.println("必须输入客户端id");
                        continue;
                    }
                    int id = Integer.parseInt(line.substring(0, idx));
                    String message = line.substring(idx + 1);
                    TcpSocket socket = socketList.get(id);
                    if (socket == null) {
                        System.out.println("客户端不存在");
                    } else {
                        socket.send(message);
                    }
                } catch (NullPointerException e) {
                    System.out.println("客户端id不能为空");
                } catch (NumberFormatException e) {
                    System.out.println("客户端id非法");
                }
            }
            System.out.println("end");
        }
    }
}
