package tcp;

import handler.TcpHandler;
import handler.impl.ClientHandler;
import handler.impl.ServerHandler;
import util.Reader;

import java.io.*;
import java.net.Socket;
import java.net.SocketException;
import java.util.Date;

public class TcpSocket implements Runnable {
    private int id;
    private Socket socket;
    private String account;
    private InputStream in;
    private OutputStream out;

    private BufferedReader reader;
    private PrintStream writer;

    private TcpServer server;
    private TcpHandler handler;


    private Date startTime;
    private Date lastTime;

    private boolean isServer;
    private boolean login;
    private boolean isForward;

    TcpSocket(TcpServer server, int id, Socket socket) throws IOException {
        this.server = server;
        this.id = id;
        this.socket = socket;
        this.isServer = true;
        this.handler = new ClientHandler();
        init();
    }

    public TcpSocket(String address, int port) throws IOException {
        this.socket = new Socket(address, port);
        this.isServer = false;
        this.handler = new ServerHandler();
        init();
    }

    private void init() throws IOException {
        this.in = this.socket.getInputStream();
        this.out = this.socket.getOutputStream();

        this.reader = new BufferedReader(new InputStreamReader(in, "UTF-8"));
        this.writer = new PrintStream(out);

        this.startTime = new Date();
        this.lastTime = new Date();
    }

    @Override
    public void run() {
        try {
            String line;
            while (!socket.isClosed() && (line = reader.readLine()) != null) {
                if (isServer) {
                    handler.handler(this, line);
                } else {
                    if (!handler.handler(this, line)) {
                        break;
                    }
                }
            }
        } catch (SocketException e) {
            System.out.println("与服务器断开连接");
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            System.out.println("按下回车键继续......");
            close();
        }
    }

    public int getId() {
        return id;
    }

    public Date getStartTime() {
        return startTime;
    }

    public void setStartTime(Date startTime) {
        this.startTime = startTime;
    }

    public Date getLastTime() {
        return lastTime;
    }

    public void setLastTime(Date lastTime) {
        this.lastTime = lastTime;
    }

    Socket getSocket() {
        return socket;
    }

    public boolean isLogin() {
        return login;
    }

    public void setLogin(boolean login) {
        this.login = login;
    }

    public boolean isForward() {
        return isForward;
    }

    public void setForward(boolean forward) {
        isForward = forward;
    }

    public void send(String message) {
        try {
            if (!socket.isClosed()) {
                if (!"".equals(message)) {
                    writer.write(message.getBytes("UTF-8"));
                    writer.println();
                    writer.flush();
                } else {
                    System.out.println("消息不能为空");
                }
            } else {
                System.out.println("与服务器断开连接");
            }
        } catch (UnsupportedEncodingException e) {
            System.out.println("不支持的编码格式:" + e.getMessage());
        } catch (IOException e) {
            System.out.println("连接异常:" + e.getMessage());
        }
    }

    /**
     * 客户端方法 用于读取键盘输入并向服务器发送信息
     * <p>
     * 注意：客户端必须手动执行该方法，否则无法向服务器发送信息
     * 服务器绝不能执行该方法，服务器的读键盘操作由TcpServer类中新开线程执行
     */
    public void write() {
        String line;
        Reader reader = new Reader();
        while (!socket.isClosed() && (line = reader.readKB()) != null) {
            send(line);
            if ("exit".equals(line)) {
                close();
            }
        }
    }

    private void close() {
        if (server != null && isServer) {
            server.removeClient(this);
        }
        try {
            if (in != null) {
                in.close();
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        try {
            if (out != null) {
                out.close();
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        try {
            if (socket != null) {
                socket.close();
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
